/**
 * osd_userinterface_data_songLyrics.h: Presents lyrics of the current song
 * 
 * Copyright: See COPYING file that comes with this distribution
 *
 * Author: Matthias Feistel <hitman_47@users.sourceforge.net>, (C) 2007
 */

#include <vector>
#include <vdr/interface.h>
#include <vdr/menu.h>
#include <vdr/plugin.h>
#include "osd_userinterface_data_songLyrics.h"
#include "settings.h"
#include "web_metadata.h"
#include "web_networkcontroller.h"
#include "ext/xmlParser/xmlParser.h"  /** in order to parse songLyrics from lyricwiki.org */
#include <sstream> /** url_stream */
#include "utils.h" /** rfc1738encode */
#include "web_serviceconnector.h" /** Meta() */

cUserInterface_Data_songLyrics::cUserInterface_Data_songLyrics( void ) :
	cMenuText( "", "")
{
	cSettings::GetInstance().SetOsd_Path( "songLyrics" );
	// OSD-Elemente aufbauen
	BuildItems( );
}

cUserInterface_Data_songLyrics::~cUserInterface_Data_songLyrics( )
{
	cSettings::GetInstance().SetOsd_Path( "data" );
}

void cUserInterface_Data_songLyrics::BuildItems( void )
{
	cMetaData meta;
	cNetworkController net;
	std::string buffer_HttpGetRequest;
	std::stringstream url_stream, sstream_tmp;
	std::string artist, track;
	
	std::string response( "no lyrics found" );  /** at first, assume there is no data */

	
	/** harvest the now playing metadata */
	cWebServiceConnector WebServiceConnector;
	meta = WebServiceConnector.Meta( );

	artist = meta.mp_MetadataMap[ "artist" ];
	track = meta.mp_MetadataMap[ "track" ];	     

	
	/** get that data from lyricwiki */
	url_stream << "http://lyricwiki.org/api.php?fmt=xml" <<
		"&artist=" << cUtils::rfc1738_encode( artist ) <<   /** before sendine request, tagstrings will be converted for rfc1738 compliancy */
		"&song=" << cUtils::rfc1738_encode( track );

	net.HttpGetRequest( buffer_HttpGetRequest, url_stream.str() );

	
	/** parsing received xml data: */
	XMLNode xMainNode, xNode;
	XMLResults xe;

	xMainNode = XMLNode::parseString( buffer_HttpGetRequest.c_str( ), "", &xe );

	if( !xe.error )
	{
		if( !xMainNode.getChildNode( "LyricsResult" ).isEmpty() )
		{
			// can be empty:
			if( xMainNode.getChildNode( "LyricsResult" ).getChildNode("lyrics").nElement() )
				response = xMainNode.getChildNode( "LyricsResult" ).getChildNode("lyrics").getText( );
		}
	}


	/** Now, show the data on OSD: */
	sstream_tmp.str("");
	sstream_tmp.clear();
	sstream_tmp << artist << " - " << track;
	SetTitle( sstream_tmp.str().c_str() );
	SetText( response.c_str( ) );
	Display( );
}

eOSState cUserInterface_Data_songLyrics::ProcessKey( eKeys Key )
{
	eOSState state = cMenuText::ProcessKey( Key );
	if( !strcmp( cSettings::GetInstance().GetOsd_Path( ), "artistMetadata" ) )
	{
		switch( Key )
		{
		case kOk:
			return osContinue;
		default:
			break;
		}
	}

	return state;
}
